package eu.dnetlib.data.collector.plugins.sftp;

import java.util.Set;

import com.google.common.base.Splitter;
import com.google.common.collect.Sets;
import eu.dnetlib.rmi.data.CollectorServiceException;
import eu.dnetlib.rmi.data.InterfaceDescriptor;
import eu.dnetlib.rmi.data.plugin.AbstractCollectorPlugin;

/**
 * Created by alessia on 15/03/17.
 */
public abstract class AbstractSftpCollectorPlugin extends AbstractCollectorPlugin {

	private SftpIteratorFactory sftpIteratorFactory;

	@Override
	public Iterable<String> collect(final InterfaceDescriptor interfaceDescriptor, final String fromDate, final String toDate)
			throws CollectorServiceException {
		final String baseUrl = interfaceDescriptor.getBaseUrl();
		final String recursive = interfaceDescriptor.getParams().get("recursive");
		final String extensions = interfaceDescriptor.getParams().get("extensions");

		if ((baseUrl == null) || baseUrl.isEmpty()) {
			throw new CollectorServiceException("Param 'baseurl' is null or empty");
		}
		if ((recursive == null) || recursive.isEmpty()) {
			throw new CollectorServiceException("Param 'recursive' is null or empty");
		}
		if ((extensions == null) || extensions.isEmpty()) {
			throw new CollectorServiceException("Param 'extensions' is null or empty");
		}
		if (fromDate != null && (!fromDate.matches("\\d{4}-\\d{2}-\\d{2}(T\\d{2}:\\d{2}:\\d{2}Z)?+") && !fromDate.matches("\\d{4}-\\d{2}-\\d{2}") )) {
			throw new CollectorServiceException("Invalid date (yyyy-MM-dd'T'HH:mm:ss'Z' or yyyy-MM-dd ): " + fromDate);
		}

		Set<String> extensionSet = parseSet(extensions);

		return getIterable(interfaceDescriptor, baseUrl, Boolean.parseBoolean(recursive), extensionSet, fromDate);
	}

	protected abstract Iterable<String> getIterable(final InterfaceDescriptor interfaceDescriptor, final String baseUrl, final boolean isRecursive, final Set<String> extensionSet, final String fromDate ) throws CollectorServiceException;

	private Set<String> parseSet(final String extensions) {
		return Sets.newHashSet(Splitter.on(",").omitEmptyStrings().trimResults().split(extensions));
	}

	public SftpIteratorFactory getSftpIteratorFactory() {
		return sftpIteratorFactory;
	}

	public void setSftpIteratorFactory(SftpIteratorFactory sftpIteratorFactory) {
		this.sftpIteratorFactory = sftpIteratorFactory;
	}

}
